--------------------------------------------
-- Object: Map
-- Descr: A map containing tiles
-- Parent: -
--------------------------------------------

Map = CreateClass();

function Map:new(map)
	-- Map data
	local tilesetImage = engine.graphics.newImage("resources/images/tileset.png");
	tilesetImage:setFilter("nearest", "nearest");
	local tilesetImageW = tilesetImage:getWidth();
	local tilesetImageH = tilesetImage:getHeight();
	local tilesetQuads = {};
	local nQuads = 0;

	-- Object data
	self.tileSize = game.maps.metadata.tileSize;
	self.widthInTiles = map.info.widthInTiles;
	self.heightInTiles = map.info.heightInTiles;
	self.spriteBatch = engine.graphics.newSpriteBatch(tilesetImage, map.info.widthInTiles * map.info.heightInTiles, "dynamic");
	self.tilemap = {};

	-- Convert tileset to quads
	for y=0, (tilesetImageH / self.tileSize)-1 do
		for x=0, (tilesetImageW / self.tileSize)-1 do
			tilesetQuads[nQuads] = engine.graphics.newQuad(x * self.tileSize, y * self.tileSize, self.tileSize, self.tileSize, tilesetImageW, tilesetImageH);
			nQuads = nQuads+1;
		end
	end
	
	-- Convert map parameter to 2D table
	for y=0, map.info.heightInTiles-1 do
		self.tilemap[y] = {};
		for x=0, map.info.widthInTiles-1 do
			self.tilemap[y][x] = tilesetQuads[map.tiles[map.info.widthInTiles * y + x]];
		end
	end

	self:updateTiles();
end

function Map:updateTiles()
	self.spriteBatch:clear();

	for y=0, self.heightInTiles-1 do
		for x=0, self.widthInTiles-1 do
			local quad = self.tilemap[y][x]
			self.spriteBatch:add(quad, x * self.tileSize, y * self.tileSize);
		end
	end

	self.spriteBatch:flush();
end

function Map:draw()
	engine.graphics.draw(self.spriteBatch, 0, 0, 0, 0.8, 0.8);
end

function Map:getWidthInTiles()
	return self.widthInTiles;
end

function Map:getHeightInTiles()
	return self.heightInTiles;
end