--------------------------------------------
-- Object: Player
-- Descr: The object controller by the player
-- Parent: -
--------------------------------------------

Player = CreateClass();

-- Direction ENUMs for clarity
local LEFT = 1;
local RIGHT = 2;
local UP = 3;
local DOWN = 4;

-- Idle sprite indices
local directionIndex = {
	[UP] = 10;
	[DOWN] = 1;
	[LEFT] = 4;
	[RIGHT] = 7;
}

function Player:new(x, y, spritesheet)
	self.x = x;
	self.y = y;
	self.w = game.maps.metadata.charW;
	self.h = game.maps.metadata.charH;
	self.speed = 60;
	self.dir = DOWN;
	self.spriteIndex = 1;

	-- Set sprite sheet if it's been given
	if (spritesheet) then
		self:setSpritesheet(spritesheet);
	end

	-- Move player
	hook.add("GAME_UPDATE", "move_player", function(dt)
		self:move(dt);
	end);

	-- Set player sprite to idle after animation ended
	hook.add("GAME_KEYRELEASED", "idle_player", function(key)
		if (not self:isFrozen()) then
			self.spriteIndex = directionIndex[self.dir];
		end
	end);
end

function Player:draw()
	-- Draw player if spritesheet is set
	if (self.spritesheet) then
		engine.graphics.setColor(255, 255, 255, 255);
		engine.graphics.draw(self.spritesheet.image, self.spritesheet[self.spriteIndex], self.x, self.y - (self.h / 2));
	end
end

function Player:animateMovement(dir)
	local increment = 0;
	self.dir = dir;

	-- Returns a cycle of 0, 1, 2, 3, 0...
	local counter = math.floor(timer.curTime() * 10) % 4;

	-- Walk pattern
	if (counter == 0) then
		increment = 1;
	elseif (counter == 1) then
		increment = 0;
	elseif (counter == 2) then
		increment = 2;
	else
		increment = 0;
	end

	-- Add increment to move legs
	self.spriteIndex = directionIndex[dir] + increment;
end

function Player:move(dt)
	if (not self:isFrozen()) then
		-- Calculate step size
		local diff = dt * self.speed;

		-- Decide direction
		if (engine.keyboard.isDown("up", "w")) then
			self.y = self.y - diff;
			self:animateMovement(UP);
		elseif (engine.keyboard.isDown("down", "s")) then
			self.y = self.y + diff;
			self:animateMovement(DOWN);
		elseif (engine.keyboard.isDown("left", "a")) then
			self.x = self.x - diff;
			self:animateMovement(LEFT);
		elseif (engine.keyboard.isDown("right", "d")) then
			self.x = self.x + diff;
			self:animateMovement(RIGHT);
		end
	end
end

function Player:setSpritesheet(spritesheet)

	self.spritesheet = spritesheet; -- quad array

end

function Player:setSprite(index)

	self.spriteIndex = index;

end

function Player:freeze()

	self.frozen = true;

end

function Player:unfreeze()

	self.frozen = false;

end

function Player:isFrozen()

	return self.frozen or false;

end

function Player:getPosition()

	return self.x, self.y;

end

function Player:setPosition(x, y)

	self.x = x or self.x;
	self.y = y or self.y;

end