--> Hook Library
-- Author: Skere
-- Description: Callback splitter. Not case-sensitive.
-- Dependancy: None
--------------------------------------------

local table = table;

hook = {};
hook.storage = {};

--[[---------------------------------------------------------
   Name:	hook.add
   Desc:	Adds a hook to the given event.
------------------------------------------------------------]]

function hook.add( event, identifier, callback )

	local event = string.lower( event );
	local identifier = string.lower( identifier );
	local evt = hook.storage[event];

	if ( not evt ) then

		hook.storage[event] = {};

	end

	hook.storage[event][identifier] = callback;

end

--[[---------------------------------------------------------
   Name:	hook.remove
   Desc:	Removes a specific hook.
------------------------------------------------------------]]

function hook.remove( event, identifier )

	local event = string.lower( event );
	local identifier = string.lower( identifier );

	if ( not hook.storage[event] ) then

		return nil;

	end

	hook.storage[event][identifier] = nil;

end

--[[---------------------------------------------------------
   Name:	hook.Call
   Desc:	Calls an event with the given arguments.
------------------------------------------------------------]]

function hook.call( event, ... )

	local event = string.lower( event );
	local evt = hook.storage[event];

	if ( not evt ) then

		return nil;

	end

	for _, callback in pairs( evt ) do

		callback( ... );

	end

end

--[[---------------------------------------------------------
   Name:	hook.emptyEvent
   Desc:	Removes all hooks from the given event.
------------------------------------------------------------]]

function hook.emptyEvent( event )

	hook.storage[event] = nil;

end