--> Math Library extension
-- Author: Skere
-- Description: Adds more functions to the default math library.
-- Dependancy: None
--------------------------------------------

local _math = math;

--[[---------------------------------------------------------
   Name:	math.constrain
   Desc:	Clamps a number value between a minimum and a maximum.
------------------------------------------------------------]] 

function math.constrain( value, min, max )

	if ( value < min ) then

		return min;

	elseif ( value > max ) then

		return max;

	else

		return value;

	end

end

--[[---------------------------------------------------------
   Name:	math.distance
   Desc:	Calculates the 2D difference between two points.
------------------------------------------------------------]] 

function math.distance( x1, y1, x2, y2 )

	local calc = ( ( x2 - x1 ) ^ 2 + ( y2 - y1 ) ^ 2 )^0.5;

	return calc;

end

--[[---------------------------------------------------------
   Name:	math.Round
   Desc:	Rounds the number based off the given decimals.
------------------------------------------------------------]]

function math.round( value, dec )

	local dec = dec or 0;
	local multi = 10 ^ dec;

	local calc = _math.floor( value * multi + 0.5 ) / multi;

	return calc;

end

--[[---------------------------------------------------------
   Name:	math.angle
   Desc:	Returns the angle between two 2D positions.
------------------------------------------------------------]]

function math.angle( x1, y1, x2, y2 )

	return _math.atan2( y2 - y1, x2 - x1 ) * 180 / math.pi;

end

--[[---------------------------------------------------------
   Name:	math.between
   Desc:	Decides whether the value is between the given values.
------------------------------------------------------------]]

function math.between( value, side_1 , side_2 )

	local max = max or _math.huge;

	if ( ( value >= side_1 ) and ( value <= side_2 ) ) then

		return true;

	elseif ( ( value <= side_1 ) and ( value >= side_2 ) ) then

		return true;

	else

		return false;

	end

end