--> Table Library
-- Author: Skere
-- Description: Extends the default table library.
-- Dependancy: None
--------------------------------------------

--[[---------------------------------------------------------
   Name:	table.print
   Desc:	Prints the full table, including table children
------------------------------------------------------------]]

local function getKeys( tab )

	local keys = {}
	local id = 1

	for k, v in pairs( tab ) do

		keys[ id ] = k
		id = id + 1

	end

	return keys

end

function table.print( t, indent, done )

	done = done or {}
	indent = indent or 0
	local keys = getKeys( t )

	for i = 1, #keys do
		local key = keys[ i ]
		local value = t[ key ]
		io.write( string.rep( "\t", indent ) )

		if  ( type(value)=="table" and not done[ value ] ) then

			done[ value ] = true
			io.write( tostring( key ) .. ":" .. "\n" )
			table.print( value, indent + 2, done )
			done[ value ] = nil

		else

			io.write( tostring( key ) .. "\t=\t" )
			io.write( tostring( value ) .. "\n" )

		end

	end

end

--[[---------------------------------------------------------
   Name:	table.Copy
   Desc:	Returns a quick copy of the table
------------------------------------------------------------]]

function table.copy( tab )

	local result = { unpack( tab ) };

	return result;

end

--[[---------------------------------------------------------
   Name:	table.DeepCopy
   Desc:	Returns a deep copy of the table
------------------------------------------------------------]]

local function copy( value )

	local valueType = type( value );

	if ( valueType == "table" ) then

		return table.deepCopy( value );

	else

		return value;

	end

end

function table.deepCopy( tab )

	local result = {};

	for k, v in pairs( tab ) do

		result[k] = copy( v );

	end

	setmetatable( result, copy( getmetatable( tab ) ) );

	return result;

end

--[[---------------------------------------------------------
   Name:	table.contains
   Desc:	Checks whether the table contains the given value
------------------------------------------------------------]]

function table.contains( tab, value )

	for _, v in pairs( tab ) do

		if ( v == value ) then

			return true;

		end

	end

	return false;

end

--[[---------------------------------------------------------
   Name:	table.size
   Desc:	Calculates the table size of a table with numberic indexes.
------------------------------------------------------------]]

function table.size(tab)
	local result = 0;

	for k, v in ipairs(tab) do
		result = result + 1;
	end

	return result;
end