--> Timer Library
-- Author: Skere
-- Description: Adds timers with callbacks similarly to the Garry's Mod library.
-- Dependancy: Love2D game engine, hook library
--------------------------------------------

-- Optimalisation
local pairs = pairs;
local table = table;
local math = math;

timer = {};

local t = 0;

local u_timer = {};
local n_timer = {};

--[[---------------------------------------------------------
   Name:	timer.tick
   Desc:	Ticks the timer. Should only be called in update.
------------------------------------------------------------]]

function timer.tick( dt )

	t = ( dt + t );

	for k, v in pairs( u_timer ) do

		v[1] = v[1] - dt;

		if ( v[1] <= 0 ) then

			v[2]();
			u_timer[k] = nil;

		end

	end

	for k, v in pairs( n_timer ) do

		v[1] = v[1] - dt;

		if ( v[1] <= 0 ) then

			v[4]();
			v[1] = v[2]
			v[3] = v[3] - 1;

			if ( v[3] <= 0 ) then

				n_timer[k] = nil;

			end

		end

	end

end

--[[---------------------------------------------------------
   Name:	timer.curTime
   Desc:	Returns the uptime in seconds.
------------------------------------------------------------]]

function timer.curTime()

	return t;

end

--[[---------------------------------------------------------
   Name:	timer.simple
   Desc:	Adds a simple delay to the given function.
------------------------------------------------------------]]

function timer.simple( delay, func )

	table.insert( u_timer, { delay, func } );

end

--[[---------------------------------------------------------
   Name:	timer.create
   Desc:	Creates a timer with more functionality.
------------------------------------------------------------]]

function timer.create( id, delay, rep, func )

	if ( rep == 0 ) then

		rep = math.huge;

	end

	n_timer[id] = {delay, delay, rep, func};

end

--[[---------------------------------------------------------
   Name:	timer.timeLeft
   Desc:	Returns the time left for the next callback of a timer.
------------------------------------------------------------]]

function timer.timeLeft( id )

	if ( not n_timer[id] ) then

		return nil;

	end

	return n_timer[id][2];

end

--[[---------------------------------------------------------
   Name:	timer.delete
   Desc:	Deletes the given timer.
------------------------------------------------------------]]

function timer.delete( id )

	n_timer[id] = nil;

end


--[[---------------------------------------------------------
   Name:	timer.exists / timer.exist
   Desc:	Checks whether the timer exists.
------------------------------------------------------------]]

function timer.exists( id )

	return ( n_timer[id] ~= nil );

end

timer.exist = timer.exists; -- alias

--[[---------------------------------------------------------
   Name:	timer.GetAll
   Desc:	Returns a table with all timer ids.
------------------------------------------------------------]]

function timer.getAll()

	local result = {};
	local count = 1;

	for k, _ in pairs( n_timer ) do

		result[count] = k;
		count = count + 1;

	end

	return result;

end

hook.add("GAME_UPDATE", "timer_lib", function(dt)
	timer.tick(dt);
end)