--[[
	Main
	 - Engine hook callbacks
	 - Module management
--]]

engine = love;

-- Includes all global lua files
local files = {
	-- Load libraries (except json)
	"libraries.general";
	"libraries.math";
	"libraries.table";
	"libraries.hook";
	"libraries.string";
	"libraries.timer";
	"libraries.draw";
	"libraries.oop";
	"libraries.json";

	-- Load source files
	"game.functions";
	"game.datastorage";
	"game.statemachine";
	"game.renderer";
	
	"game.assets.maps";

	"game.objects.Player";
	"game.objects.World";
	"game.objects.Map";
}

-- Loads all globally required files
local function prepare()
	print("Game version " .. game.config.version);

	-- Game files
	print("Loading source files...");
		for _, v in ipairs(files) do

			print("  - Including source file: " .. v);
			require(v);

		end
	print("Source files loaded successfully!");

	-- Addon files
	print("Loading addons...");
		local addons = engine.filesystem.getDirectoryItems("addons");

		for _, addon in ipairs( addons ) do
			local exists_path = "addons/" .. addon .. "/setup";
			local require_path = "addons." .. addon .. ".setup";

			if (engine.filesystem.exists(exists_path .. ".lua")) then

				print("  - Including addon: " .. addon);
				require(require_path);

			else

				print("Couldn't find setup.lua for " .. addon);

			end
		end
	print("Addons loaded successfully!");

	-- Savedata
	print("Loading save data...");
		data:load();
	print("Save data loaded succesfully!");

	print("All files have been initialized.");
	print("\nDeveloper console:");

	love.graphics.setDefaultFilter("nearest", "nearest");
end

-- Important global functions

function width()

	return engine.graphics.getWidth();

end

function height()

	return engine.graphics.getHeight();

end

-- Add hooks to important callbacks

function engine.load()
	prepare();

	hook.call("GAME_LOAD");
end

function engine.update( dt )

	hook.call( "GAME_UPDATE", dt );

end

function engine.draw()
 love.graphics.print("FPS: "..love.timer.getFPS(), 10, 20) -- temp
	hook.call( "GAME_DRAW" );

end

function engine.mousepressed(x, y, button, touch)

	hook.call("GAME_MOUSEPRESSED", x, y, button, touch);

end

function engine.mousereleased(x, y, button, touch)

	hook.call("GAME_MOUSERELEASED", x, y, button, touch);

end

function engine.keypressed(key)

	hook.call("GAME_KEYPRESSED", key);

end

function engine.keyreleased(key)

	hook.call("GAME_KEYRELEASED", key);

end

function engine.focus(f)

	hook.call("GAME_FOCUS", f);

end

function engine.quit()
	
	hook.call("GAME_QUIT");

end